//
//  MXChat.h
//  ChatSDK
//
//  Copyright © 2016 moxtra. All rights reserved.
//

#import <Foundation/Foundation.h>

NS_ASSUME_NONNULL_BEGIN

@class MXUserItem;
@class MXChatCategory;

/**
 The MXChat class provides interfaces for basic information about chats as well as some lightweight operations.
 */
@interface MXChat : NSObject

/**
 The id of this chat.
 */
@property (nonatomic, strong, readonly) NSString *chatId;

/**
 Topic of this chat.
 */
@property (nonatomic, strong, readonly, nullable) NSString *topic;

/**
 A boolean value indicating whether this chat is an individual one.
 */
@property (nonatomic, assign, readonly) BOOL isIndividualChat;

/**
 A boolean value indicating whether this chat is favorite.
 */
@property (nonatomic, assign, readonly,getter=isFavorite) BOOL favorite;

/**
 A boolean value indicating whether this chat has been accpeted.
 */
@property (nonatomic, assign, readonly, getter=isAccepted) BOOL accepted;

/**
 The tags of this chat.
 */
@property (nonatomic, strong, readonly, nullable) NSDictionary<NSString *, NSString *>* tags;

/**
 The owner of the chat.
 */
@property (nonatomic, assign, readonly) MXUserItem *owner;

/**
 When the chat was created.
 */
@property (nonatomic, strong, readonly) NSDate *createdTime;

/**
 When the last feed was posted.
 */
@property (nonatomic, strong, readonly) NSDate *lastFeedTime;

/**
 The count of unread feeds in this chat.
 */
@property (nonatomic, assign, readonly) UInt64 unreadFeedsCount;

/**
 The count of uncomplete todos in this chat.
 */
@property (nonatomic, assign, readonly) UInt64 uncompleteTodosCount;

/**
 The count of todos that are assigned to the current user.
 */
@property (nonatomic, assign, readonly) UInt64 assignToMeTodosCount;

/**
 The count of all todos in this chat.
 */
@property (nonatomic, assign, readonly) UInt64 todosCount;

/**
 The content of the last feed.
 */
@property (nonatomic, strong, readonly, nullable) NSString *lastFeedContent;

/**
 The user who create the last feed.
 */
@property (nonatomic, strong, readonly, nullable) MXUserItem *lastFeedUser;

/**
 The category of this chat.
 */
@property (nonatomic, strong, readonly, nullable) MXChatCategory *category;

/**
 A boolean value indicating whether this chat is muted by the user.
 */
@property (nonatomic, assign, readonly) BOOL isMute;

/**
 Total sign files count.
 */
@property (nonatomic, assign, readonly) UInt64 signFilesCount;

/**
 The count of sign files that are your turn to sign.
 */
@property (nonatomic, readonly) UInt64 myTurnSignFilesCount;

/**
 Update topic of this chat.
 
 @param topic             New topic to be set.
 @param completionHandler A block to be executed when the action completes.
 @discussion A user with viewer access type or if this chat is a individual chat, it has no permission for updating topic. An error with code MXChatSDKErrorNoPermission will occurred if you do so.
 */
- (void)updateTopic:(NSString *)topic withCompletionHandler:(void (^ _Nullable)(NSError * _Nullable errorOrNil))completionHandler;

/**
 Update category of this chat by specifying a new category.
 
 @param category          The new category that the chat will be assigned to.
 @param completionHandler A block to be executed when the action completes.
 @discussion A user with viewer access type has no permission for updating category. An error MXChatSDKErrorNoPermission will occurred if you do so.
 */
- (void)updateCategory:(MXChatCategory *)category withCompletionHandler:(void (^_Nullable)(NSError * _Nullable errorOrNil))completionHandler;

/**
 Update tags represented by a dictionary for this chat.
 
 @param tags              A dictionary representing tags that are going to be set.
 @param completionHandler A block object to be executed when the action completes
 @discussion You can use this interface to set custom tags for chats so that these tags can be fetched even if you have ever unlinked.
 */
- (void)updateTags:(nullable NSDictionary<NSString *, NSString *> *)tags withCompletionHandler:(void (^_Nullable)(NSError * _Nullable errorOrNil))completionHandler;

/**
 Update cover of this chat by specifying an image's file path.
 
 @param imagePath         The file path of a image that will be used as cover
 @param completionHandler A block object to be executed when the action completes
 @discussion A user with viewer access type has no permission for updating cover. An error with code MXChatSDKErrorNoPermission will occurred if you do so.
 */
- (void)updateCoverWithImagePath:(nullable NSString *)imagePath withCompletionHandler:(void (^_Nullable)(NSError * _Nullable errorOrNil))completionHandler;

/**
 Fetch cover of this chat
 
 @param completionHandler A block object to be executed when the action completes
 */
- (void)fetchCoverWithCompletionHandler:(void (^_Nullable)(NSError * _Nullable errorOrNil, NSString * _Nullable localPathOrNil))completionHandler;

/**
 Mark this chat as read to clear unread feed count.
 
 @param completionHandler A block object to be executed when the action completes
 */
- (void)markAsReadWithCompletionHandler:(void (^_Nullable)(NSError * _Nullable errorOrNil))completionHandler;

/**
 Favorite or un-favorite this chat.

 @param isFavorite Set YES to mark this chat as favorite, otherwise set to NO.
 @param completionHandler A block object to be executed when the action completes
 */

- (void)markAsFavorite:(BOOL)isFavorite withCompletionHandler:(void (^_Nullable)(NSError * _Nullable errorOrNil))completionHandler;
/**
 Mute or unmute this chat.
 
 @param mute              Set YES if this will be muted, otherwise NO.
 @param completionHandler A block object to be executed when the action completes
 */
- (void)mute:(BOOL)mute withCompletionHandler:(void (^_Nullable)(NSError * _Nullable errorOrNil))completionHandler;

@end

NS_ASSUME_NONNULL_END
