//
//  MXChatClient.h
//  ChatSDK
//
//  Copyright © 2016 moxtra. All rights reserved.
//

#import <Foundation/Foundation.h>

NS_ASSUME_NONNULL_BEGIN

@class MXLinkConfig;
@class MXChatConfig;
@class MXMeetConfig;
@class MXUserItem;
@class MXAuditEvent;

@protocol MXChatClientDelegate;
@protocol MXChatClientAuditDelegate;

/**
 The MXChatClient class provides interfaces for user authentication. A MXChatClient object maintains the linked user profile represented by a MXUserItem object as well as configuration objects that define chat and meet session behaviours.
 */
@interface MXChatClient : NSObject

/**
 A configuration object for all chat sessions.
 */
@property(nonatomic, strong, readonly) MXChatConfig *chatSessionConfig;

/**
 A configuration object for all meet sessions.
 */
@property(nonatomic, strong, readonly) MXMeetConfig *meetSessionConfig;

/**
 A user item object representing the linked user.
 */
@property(nonatomic, strong, readonly, nullable) MXUserItem *currentUser;

/**
 The object that acts as the delegate of the chat client.
 */
@property(nonatomic, weak, nullable) id<MXChatClientDelegate> delegate;

/**
 The object that acts as the audit delegate of the chat client.
 @discussion Make sure assign this 'auditDelegate' after login succeed.
 */
@property(nonatomic, weak, nullable) id<MXChatClientAuditDelegate> auditDelegate;

/**
 Get chat client singleton
 */
+ (instancetype)sharedInstance;

/**
 Get SDK version.
 */
+ (nonnull NSString *)version;

/**
 Set up domains before link
 
 @param baseDomain        The server domain url. There are 2 basedomains you might to use:
                         "www.moxtra.com"  for production environment,
                         "sandbox.moxtra.com" for sanbox environment
                         During development, you will point your app at Moxtra's "sandbox" development environment. Set the base domain to 'production' when pointing your app to production environment.
 @param httpsDomain       The server https domain url.
 @param wssDomain         The server socket domain url.
 @param linkConfig        The optional link configuration.
 */
- (void)setupDomain:(nullable NSString *)baseDomain
        httpsDomain:(nullable NSString *)httpsDomain
          wssDomain:(nullable NSString *)wssDomain
         linkConfig:(nullable MXLinkConfig *)linkConfig;

/**
 Link with access token. It need OAuth login with Moxtra Account.
 @discussion              Please call 'setupDomain: httpsDomain: wssDomain:linkConfig:' first before you link
 
 @param token             Access token used for authentication
 @param completionHandler A block object to be executed when the action completes
 */
- (void)linkWithAccessToken:(NSString *)token
          completionHandler:(void (^ _Nullable)(NSError * _Nullable errorOrNil))completionHandler;

/**
 Link with a unique identifier then call a handler upon completion.
 @discussion              Please call 'setupDomain: httpsDomain: wssDomain:linkConfig:' first before you link
 
 @param uniqueId          Unique ID
 @param orgId             Organization ID. Set nil if there is no organization
 @param clientId          Client ID of your app
 @param clientSecret      Client secret of your app
 @param completionHandler A block object to be executed when the action completes
 */
- (void)linkWithUniqueId:(NSString *)uniqueId
                   orgId:(NSString * _Nullable)orgId
                clientId:(NSString *)clientId
            clientSecret:(NSString *)clientSecret
       completionHandler:(void(^ _Nullable)(NSError * _Nullable errorOrNil))completionHandler;

/**
 Check if moxtra account is linked.
 
 @return True if account is linked, false if not linked or sdk not initialized.
 */
- (BOOL)isLinked;

/**
 Unlink current account.
 */
- (void)unlink;

/**
 Update the current user's firstname and lastname.
 
 @param firstname         The firstname to set. Set nil if you don't want to update firstname.
 @param lastname          The lastname to set. Set nil if you don't want to update lastname.
 @param completionHandler A block object to be executed when the action completes.
 */
- (void)updateCurrentUserProfileWithFirstname:(nullable NSString *)firstname
                                     lastname:(nullable NSString *)lastname
                            completionHandler:(void (^ _Nullable)(NSError * _Nullable errorOrNil))completionHandler;
/**
 Update the current user's avatar.
 
 @param avatar            An image object that is going to be set as the current user's avatar.
 @param completionHandler A block object to be executed when the action completes.
 */
- (void)updateCurrentUserProfileAvatar:(UIImage *)avatar
                 withCompletionHandler:(void (^ _Nullable)(NSError * _Nullable errorOrNil))completionHandler;

/**
 Tells the Chat SDK that Apple Push Services successfully completes the registration process.
 
 @param token A token that identifies the device to Apple Push Notification Service (APNS)
 */
- (void)registerNotificationDeviceToken:(NSData *)token;

/**
 Return a boolean value that indicates whether the specified notification is a remote notification from Moxtra
 
 @param userInfo The notification dictionary
 @return YES if it is a remote notification from Moxtra, otherwise NO.
 */
- (BOOL)isMoxtraRemoteNotification:(NSDictionary *)userInfo;

/**
 Parse a specified remote notification to get the related item.
 
 @param userInfo          The notification dictionary.
 @param completionHandler A block object to be executed when the action completes.
 @discussion The item parameter passed to completionHandler can be an instance of MXChat, MXMeet or nil.
 */
- (void)fetchItemWithRemoteNotification:(NSDictionary *)userInfo completionHandler:(void (^ _Nullable)(NSError * _Nullable errorOrNil, id _Nullable item))completionHandler;

#pragma mark - Unavailable

- (instancetype)init UNAVAILABLE_ATTRIBUTE;
+ (instancetype)new UNAVAILABLE_ATTRIBUTE;

@end


/**
 The MXChatClientDelegate protocol defines methods that are called by the client object in response to important events in the lifetime of your client.
 */
@protocol MXChatClientDelegate <NSObject>

@optional

/**
 Tells the delegate that the current client has logged out.
 
 @param chatClient The chat client
 
 @discussion Typical happens when you call unlink or access token expired.
 */
- (void)chatClientDidUnlink:(MXChatClient *)chatClient;

/**
 Tells the delegate that the user's storage has exceeded permission.
 
 @param chatClient The chat client
 */
- (void)chatClientDidStorageBeyondPermission:(MXChatClient *)chatClient;

@end

/**
 The MXChatClientAuditDelegate protocol defined the method that is called when any audit events releated with current user and current client has been happened.
 */
@protocol MXChatClientAuditDelegate <NSObject>

@optional

/**
 Tells the delegate that the current client has received a audit event.
 
 @param auditEvent The reveived audit event
 */
- (void)chatClientDidReceivedAuditEvent:(MXAuditEvent *)auditEvent;

@end

NS_ASSUME_NONNULL_END


