//
//  MXChatListModel.h
//  ChatSDK
//
//  Copyright © 2017 moxtra. All rights reserved.
//

#import <Foundation/Foundation.h>

NS_ASSUME_NONNULL_BEGIN

@class MXChatListModel;
@class MXChat;
@class MXChatCategory;


/**
 The MXChatListModelDelegate protocol describes the methods that MXChatListModel objects call on their delegates to handle chat list events.
 */
@protocol MXChatListModelDelegate <NSObject>

@optional

/**
 Tells the delegate that some chats have been created.
 
 @param chatListModel The related chat list model.
 @param createdChats  An array containing created chats.
 */
- (void)chatListModel:(MXChatListModel *)chatListModel didCreateChats:(NSArray<MXChat *> *)createdChats;

/**
 Tells the delegate that some chats have been updated.
 
 @param chatListModel The related chat list model.
 @param updatedChats  An array containing updated chats.
 */
- (void)chatListModel:(MXChatListModel *)chatListModel didUpdateChats:(NSArray<MXChat *> *)updatedChats;

/**
 Tells the delegate that some chats have been deleted.
 
 @param chatListModel The related chat list model.
 @param deletedChats  An array containing deleted chats.
 */
- (void)chatListModel:(MXChatListModel *)chatListModel didDeleteChats:(NSArray<MXChat *> *)deletedChats;

/**
 Tells the delegate that some categories have been created.
 
 @param chatListModel     The related chat list model
 @param createdCategories An array containing created categories
 */
- (void)chatListModel:(MXChatListModel *)chatListModel didCreateCategories:(NSArray<MXChatCategory *> *)createdCategories;

/**
 Tells the delegate that some categories have been updated.
 
 @param chatListModel     The related chat list model
 @param updatedCategories An array containing updated categories
 */
- (void)chatListModel:(MXChatListModel *)chatListModel didUpdateCategories:(NSArray<MXChatCategory *> *)updatedCategories;

/**
 Tells the delegate that some categories have been deleted.
 
 @param chatListModel     The related chat list model
 @param deletedCategories An array containing deleted categories
 */
- (void)chatListModel:(MXChatListModel *)chatListModel didDeleteCategories:(NSArray<MXChatCategory *> *)deletedCategories;

@end


/**
 The MXChatListModel class provides interfaces for creating, deleting, accepting and declining chats. It also maintains all the chats object related with current user.
 */
@interface MXChatListModel : NSObject

/**
 The object that acts as delegate of the chat list.
 */
@property (nonatomic, weak, nullable) id<MXChatListModelDelegate> delegate;

/**
 Return an array containing all chats.
 */
- (NSArray<MXChat *> *)chats;

/**
 Get a MXChat instance via its chat id

 @param chatId  The id of this chat.
 @return MXChat instance
 */
- (MXChat *)getChatsById:(NSString *)chatId;

/**
 Return an array containing all chat categories.
 */
- (NSArray<MXChatCategory *> *)categories;

/**
 Query chats of a specified category.
 
 @param category The related category.
 @return An array containing all chats of the specified category.
 */
- (NSArray<MXChat *> *)chatsInCategory:(MXChatCategory *)category;

/**
 Create a group chat with the specified topic.
 
 @param topic             The topic of the created chat
 @param completionHandler A block object to be executed when the action completes
 @discussion If the number of chats related with the user has reached the upper limit, an error with code MXChatSDKErrorExceedLimit will occurred.
 */
- (void)createChatWithTopic:(NSString * _Nullable)topic
          completionHandler:(void (^ _Nullable)(NSError * _Nullable errorOrNil, MXChat * _Nullable chat))completionHandler;

/**
 Create a category with the specified name.
 
 @param categoryName      The category name.
 @param completionHandler A block object to be executed when the action completes.
 */
- (void)createCategoryWithName:(NSString *)categoryName
             completionHandler:(void (^ _Nullable)(NSError * _Nullable errorOrNil, MXChatCategory * _Nullable category))completionHandler;

/**
 Update a category with the specified name.
 
 @param category          The category need to update
 @param categoryName      The category name.
 @param completionHandler A block object to be executed when the action completes.
 */
- (void)updateCategory:(MXChatCategory *)category
              withName:(NSString *)categoryName
             completionHandler:(void (^ _Nullable)(NSError * _Nullable errorOrNil))completionHandler;

/**
 Delete a category
 
 @param category          The category need to delete
 @param completionHandler A block object to be executed when the action completes.
 */
- (void)deleteCategory:(MXChatCategory *)category
     withCompletionHandler:(void (^ _Nullable)(NSError * _Nullable errorOrNil))completionHandler;

/**
 Start an individual chat with a user by specifying unique id and org id.
 
 @param uniqueId          The related user's unique id.
 @param orgId             The related user's org id which could be nil.
 @param completionHandler A block object to be executed when the action completes.
 @discussion If the number of chats related with the user has reached the upper limit, an error with code MXChatSDKErrorExceedLimit will occurred.
 */
- (void)startIndividualChatWithUniqueId:(NSString *)uniqueId
                                  orgId:(NSString * _Nullable)orgId
                      completionHandler:(void (^ _Nullable)(NSError * _Nullable errorOrNil, MXChat * _Nullable chat))completionHandler;

/**
 Delete (if the current user is the owner) or leave a chat.
 
 @param chat              The chat that the user is going to delete or leave.
 @param completionHandler A block object to be executed when the action completes.
 */
- (void)deleteOrLeaveChat:(MXChat *)chat
    withCompletionHandler:(void (^ _Nullable)(NSError * _Nullable errorOrNil))completionHandler;

/**
 Accept a chat

 @param chat The chat that wait user to accept or decline
 @param completionHandler A block object to be excuted when the chat gets accepted
 @discussion Usually do not need to take the initiative to accept or decline a chat.
             If this chat invitations is sent from other non-mobile SDK, you may need to accept or decline depends on whether this chat has been accepted.
 */
- (void)acceptChat:(MXChat *)chat
    withCompletionHandler:(void (^ _Nullable)(NSError * _Nullable errorOrNil))completionHandler;

/**
 Decline a chat

 @param chat The chat that wait user to accept or decline
 @param completionHandler A block object to be excuted when the chat gets declined
 @discussion Usually do not need to take the initiative to accept or decline a chat.
             If this chat invitations is sent from other non-mobile SDK, you may need to accept or decline depends on whether this chat has been accepted.
 */
- (void)declineChat:(MXChat *)chat
    withCompletionHandler:(void (^ _Nullable)(NSError * _Nullable errorOrNil))completionHandler;

@end

NS_ASSUME_NONNULL_END
