//
//  MXChatSession
//  ChatSDK
//
//  Copyright © 2016 moxtra. All rights reserved.
//

#import <Foundation/Foundation.h>
NS_ASSUME_NONNULL_BEGIN

@class MXChatSession;
@class MXChat;
@class MXTodoItem;
@class MXFileItem;
@class MXSignFileItem;
@class MXFolderItem;
@class MXChatMember;
@class MXUserItem;


/**
 The MXChatSessionDelegate protocol describes the methods that MXChatSession objects call on their delegates to handle related chat events.
 */
@protocol MXChatSessionDelegate <NSObject>

@optional

/**
 Tells the delegate that some todos have been created.
 
 @param chatSession The chat session
 @param createdTodos  Created todos
 */
- (void)chatSession:(MXChatSession *)chatSession didCreateTodos:(NSArray<MXTodoItem *> *)createdTodos;

/**
 Tells the delegate that some todos have been updated.
 
 @param chatSession The chat session
 @param updatedTodos  Updated todos
 */
- (void)chatSession:(MXChatSession *)chatSession didUpdateTodos:(NSArray<MXTodoItem *> *)updatedTodos;

/**
 Tells the delegate that some todos have been deleted.
 
 @param chatSession The chat session
 @param deletedTodos  Deleted todos
 */
- (void)chatSession:(MXChatSession *)chatSession didDeleteTodos:(NSArray<MXTodoItem *> *)deletedTodos;

/**
 Tells the delegate that some files have been created.
 
 @param chatSession The chat session
 @param createdFiles  Created files
 */
- (void)chatSession:(MXChatSession *)chatSession didCreateFiles:(NSArray<MXFileItem *> *)createdFiles;

/**
 Tells the delegate that some files have been updated.
 
 @param chatSession The chat session
 @param updatedFiles  Updated files
 */
- (void)chatSession:(MXChatSession *)chatSession didUpdateFiles:(NSArray<MXFileItem *> *)updatedFiles;

/**
 Tells the delegate that some files have been deleted.
 
 @param chatSession The chat session
 @param deletedFiles  Deleted files
 */
- (void)chatSession:(MXChatSession *)chatSession didDeleteFiles:(NSArray<MXFileItem *> *)deletedFiles;

/**
 Tells the delegate that some folders have been created.
 
 @param chatSession  The chat session
 @param createdFolders Created folders
 */
- (void)chatSession:(MXChatSession *)chatSession didCreateFolders:(NSArray<MXFolderItem *> *)createdFolders;

/**
 Tells the delegate that some folders have been updated.
 
 @param chatSession  The chat session
 @param updatedFolders Updated folders
 */
- (void)chatSession:(MXChatSession *)chatSession didUpdateFolders:(NSArray<MXFolderItem *> *)updatedFolders;

/**
 Tells the delegate that some folders have been deleted.
 
 @param chatSession  The chat session
 @param deletedFolders Deleted folders
 */
- (void)chatSession:(MXChatSession *)chatSession didDeleteFolders:(NSArray<MXFolderItem *> *)deletedFolders;

/**
 Tells the delegate that some sign files have been created.
 
 @param chatSession The chat session
 @param createdFiles  Created sign files
 */
- (void)chatSession:(MXChatSession *)chatSession didCreateSignFiles:(NSArray<MXSignFileItem *> *)createdFiles;

/**
 Tells the delegate that some sign files have been updated.
 
 @param chatSession The chat session
 @param updatedFiles  Updated sign files
 */
- (void)chatSession:(MXChatSession *)chatSession didUpdateSignFiles:(NSArray<MXSignFileItem *> *)updatedFiles;

/**
 Tells the delegate that some sign files have been deleted.
 
 @param chatSession The chat session
 @param deletedFiles  Deleted sign files
 */
- (void)chatSession:(MXChatSession *)chatSession didDeleteSignFiles:(NSArray<MXSignFileItem *> *)deletedFiles;

/**
 Tells the delegate that some members have joined the chat.
 
 @param chatSession The chat session
 @param joinedMembers Joined members
 */
- (void)chatSession:(MXChatSession *)chatSession didMembersJoin:(NSArray<MXChatMember *> *)joinedMembers;

/**
 Tells the delegate that some members in the chat have updated.
 
 @param chatSession The chat session
 @param updatedMembers Updated members
 */
- (void)chatSession:(MXChatSession *)chatSession didMembersUpdate:(NSArray<MXChatMember *> *)updatedMembers;

/**
 Tells the delegate that some memebers have left the chat.
 
 @param chatSession The chat session
 @param leftMembers Left members
 */
- (void)chatSession:(MXChatSession *)chatSession didMembersLeave:(NSArray<MXChatMember *> *)leftMembers;

@end

/**
 The MXChatSession class provides interfaces for detail information about a chat which is represented by a chat instance specified in the initialization method, as well as interacting with it.
 */
@interface MXChatSession : NSObject

/**
 The related chat.
 */
@property (nonatomic, strong, readonly) MXChat *chat;

/**
 The object that acts as delegate of the chat session
 */
@property (nonatomic, weak) id<MXChatSessionDelegate> delegate;

/**
 Email address of this chat.
 */
@property (nonatomic, strong, readonly) NSString *emailAddress;

/**
 Descriptions of this chat.
 */
@property (nonatomic, strong, readonly, nullable) NSString *descriptionString;

/**
 Initialize a chat session with specified chat
 
 @param chat The related chat
 @return Return a newly initialized chat session. This method will return nil if the chat parameter is not provided.
 */
- (nullable instancetype)initWithChat:(MXChat *)chat;

/**
 Get an array of this chat users asynchronously

 @param completionHandler A block to be executed when the action completes.
 */
- (void)getUsersWithCompletionHandler:(void (^ _Nullable)(NSError * _Nullable errorOrNil, NSArray<MXChatMember *> *members))completionHandler;

/**
 Update description of this chat.
 
 @param description       New description to be set.
 @param completionHandler A block to be executed when the action completes.
 @discussion A user with viewer access type has no permission for updating description. An error with code MXChatSDKErrorNoPermission will occurred if you do so.
 */
- (void)updateDescripion:(nullable NSString *)description withCompletionHandler:(void (^ _Nullable)(NSError * _Nullable errorOrNil))completionHandler;

/**
 Invite members to join this chat.
 
 @param users             An array containing users to be invited.
 @param completionHandler A block to be executed when the action completes.
 @discussion A user with viewer access type has no permission for inviting users. An error with code MXChatSDKErrorNoPermission will occurred if you do so. When number of users in this chat has reached the upper limit, an error with code MXChatSDKErrorExceedLimit will occurred.
 */
- (void)inviteUsers:(NSArray<MXUserItem *> *)users withCompletionHandler:(void (^ _Nullable)(NSError * _Nullable errorOrNil))completionHandler;

/**
 Invite users from an organization by specifying the org id and an array containing unique ids.
 
 @param orgId             The id of the org whose users is going to be invited.
 @param uniqueIds         An array containing unique ids of invited users.
 @param completionHandler A block to be executed when the action completes.
 @discussion A user with viewer access type has no permission for inviting users. An error with code MXChatSDKErrorNoPermission will occurred if you do so. When number of users in this chat has reached the upper limit, an error with code MXChatSDKErrorExceedLimit will occurred.
 */
- (void)inviteWithOrgId:(nullable NSString *)orgId
              uniqueIds:(NSArray<NSString *> *)uniqueIds
      completionHandler:(void (^ _Nullable)(NSError * _Nullable errorOrNil))completionHandler;

/**
 Remove members from this chat
 
 @param user              An array containing members the user is going to remove.
 @param completionHandler A block to be executed when the action completes.
 @discussion A user with viewer access type has no permission for removing users. An error with code MXChatSDKErrorNoPermission will occurred if you do so.
 */
- (void)removeUser:(MXUserItem *)user withCompletionHandler:(void (^ _Nullable)(NSError * _Nullable errorOrNil))completionHandler;

/**
 Duplicate this chat

 @param newName           The duplication's new name
 @param completionHandler A block to be executed when the action completes.
 */
- (void)duplicateChatWithNewName:(NSString *)newName
               completionHandler:(void (^)(NSError * _Nullable errorOrNil, MXChat * _Nullable duplicatedChat))completionHandler;

/**
 Update a user's access type.
 
 @param user              The user whose access type will be updated.
 @param accessType        New access type of this user.
 @param completionHandler A block to be executed when the action completes.
 @discussion A user with viewer access type has no permission for updating users. An error with code MXChatSDKErrorNoPermission will occurred if you do so.
 */
- (void)updateUser:(MXUserItem *)user
    withAccessType:(MXChatMemberAccessType)accessType
 completionHandler:(void (^ _Nullable)(NSError * _Nullable errorOrNil))completionHandler;

/**
 Fetch files of a folder. If no folder is specified, files in the root directory will be fetched.
 
 @param folder            The folder object whose files will be fetched. Set nil if you want to fetch root files
 @param completionHandler A block object to be executed when the action completes
 */
- (void)fetchFilesWithFolder:(nullable MXFolderItem *)folder
           completionHandler:(void (^ _Nullable)(NSError * _Nullable errorOrNil, NSArray<MXFileItem *> * _Nullable files))completionHandler;

/**
 Fetch sub folders of a folder. If no folder is specified, folders in the root directory will be fetched.
 
 @param folder            The folder object whose sub folders will be fetched. Set nil if you want to fetch root folders
 @param completionHandler A block object to be executed when the action completes
 */
- (void)fetchFoldersWithFolder:(nullable MXFolderItem *)folder
             completionHandler:(void (^ _Nullable)(NSError * _Nullable errorOrNil, NSArray<MXFolderItem *> * _Nullable folders))completionHandler;

/**
 Fetch sign files
 
 @param completionHandler A block object to be executed when the action completes
 */
- (void)fetchSignFilesWithCompletionHandler:(void (^ _Nullable)(NSError * _Nullable errorOrNil, NSArray<MXSignFileItem *> * _Nullable signFiles))completionHandler;

/**
 Fetch todos
 
 @param completionHandler A block object to be executed when the action completes
 */
- (void)fetchTodosWithCompletionHandler:(void (^ _Nullable)(NSError * _Nullable errorOrNil, NSArray<MXTodoItem *> * _Nullable todos))completionHandler;

/**
 Complete or uncomplete a todo.
 
 @param todo              The todo to be marked.
 @param completed         YES if the todo is going to be marked as completed, otherwise NO.
 @param completionHandler A block object to be executed when the action completes.
 @discussion A user with viewer access type has no permission for completing or uncompleting todos. An error with code MXChatSDKErrorNoPermission will occurred if you do so.
 */
- (void)markTodo:(MXTodoItem *)todo completed:(BOOL)completed withCompletionHandler:(void (^ _Nullable)(NSError * _Nullable errorOrNil))completionHandler;

/**
 Flag or unflag a todo
 
 @param todo              The todo to be marked
 @param flag              YES if the todo is going to be flagged, otherwise NO
 @param completionHandler A block object to be executed when the action completes
 @discussion A user with viewer access type has no permission for flagging or unflagging todos. An error with code MXChatSDKErrorNoPermission will occurred if you do so.
 */
- (void)markTodo:(MXTodoItem *)todo flag:(BOOL)flag completionHandler:(void (^ _Nullable)(NSError * _Nullable errorOrNil))completionHandler;

/**
 Delete a Todo
 
 @param todo              Todo to be deleted
 @param completionHandler A block object to be executed when the action completes
 @discussion A user with viewer access type has no permission for deleting todos. An error with code MXChatSDKErrorNoPermission will occurred if you do so.
 */
- (void)deleteTodo:(MXTodoItem *)todo withCompletionHandler:(void (^ _Nullable)(NSError * _Nullable errorOrNil))completionHandler;

/**
 Decline to sign a file with some reason.
 
 @param signFile          The sign file to be declined.
 @param reason            The decline reason.
 @param completionHandler A block to be executed when the action completes.
 @discussion A user with viewer access type has no permission for decling sign files. An error with code MXChatSDKErrorNoPermission will occurred if you do so.
 */
- (void)declineSignFile:(MXSignFileItem *)signFile withReason:(NSString *)reason completionHandler:(void (^ _Nullable)(NSError * _Nullable errorOrNil))completionHandler;

/**
 Delete some files.
 
 @param files             An array containing all files to be deleted.
 @param completionHandler A block to be executed when the action completes.
 @discussion A user with viewer access type has no permission for deleting files. An error with code MXChatSDKErrorNoPermission will occurred if you do so.
 */
- (void)deleteFiles:(NSArray<MXFileItem *> *)files withCompletionHandler:(void (^ _Nullable)(NSError * _Nullable errorOrNil))completionHandler;

/**
 Delete a folder.
 
 @param folder            The folder to be deleted.
 @param completionHandler A block to be executed when the action completes.
 @discussion A user with viewer access type has no permission for deleting folders. An error with code MXChatSDKErrorNoPermission will occurred if you do so.
 */
- (void)deleteFolder:(MXFolderItem *)folder withCompletionHandler:(void (^ _Nullable)(NSError * _Nullable errorOrNil))completionHandler;

/**
 Rename a file.
 
 @param file              The file to be renamed.
 @param newName           The new name.
 @param completionHandler A block to be executed when the action completes.
 @discussion A user with viewer access type has no permission for renaming files. An error with code MXChatSDKErrorNoPermission will occurred if you do so.
 */
- (void)renameFile:(MXFileItem *)file withNewName:(NSString *)newName completionHandler:(void (^ _Nullable)(NSError * _Nullable errorOrNil))completionHandler;

/**
 Rename a folder.
 
 @param folder            The folder to be renamed.
 @param newName           The new name.
 @param completionHandler A block to be executed when the action completes.
 @discussion A user with viewer access type has no permission for renaming folders. An error with code MXChatSDKErrorNoPermission will occurred if you do so.
 */
- (void)renameFolder:(MXFolderItem *)folder withNewName:(NSString *)newName completionHandler:(void (^ _Nullable)(NSError * _Nullable errorOrNil))completionHandler;

/**
 Share files by getting share and download URLs.
 
 @param files             An array containing files to be shared.
 @param completionHandler A block to be executed when the action completes.
 @discussion A user with viewer access type has no permission for sharing files. An error with code MXChatSDKErrorNoPermission will occurred if you do so.
 */
- (void)shareFiles:(NSArray<MXFileItem *> *)files withCompletionHandler:(void (^ _Nullable)(NSError * _Nullable errorOrNil, NSURL * _Nullable shareURL, NSURL * _Nullable downloadURL))completionHandler;

/**
 Share folders by getting share and download URLs.
 
 @param folders           An array containing folders to be shared.
 @param completionHandler A block to be executed when the action completes.
 @discussion A user with viewer access type has no permission for sharing folders. An error with code MXChatSDKErrorNoPermission will occurred if you do so.
 */
- (void)shareFolders:(NSArray<MXFolderItem *> *)folders withCompletionHandler:(void (^ _Nullable)(NSError * _Nullable errorOrNil, NSURL * _Nullable shareURL, NSURL * _Nullable downloadURL))completionHandler;

@end

NS_ASSUME_NONNULL_END

