//
//  MXChatViewController.h
//  ChatSDK
//
//  Copyright © 2017 moxtra. All rights reserved.
//

#import <UIKit/UIKit.h>

NS_ASSUME_NONNULL_BEGIN

@class MXChat;
@class MXUserItem;
@class MXFileItem;
@class MXChatTab;
@class MXChatViewController;

/**
 The MXChatViewControllerDelegate protocol describes the methods that MXChatViewController object call on their delegates to handle chat view events.
 */
@protocol MXChatViewControllerDelegate <NSObject>

@optional
/**
 Tells the delegate that user selected a tab
 
 @param controller The MXChatViewController
 @param index The index of tab which user selected
 */
- (void)chatViewController:(MXChatViewController *)controller didSelectTab:(NSUInteger)index;
@end

/**
 An enumeration for tabs in chats.
 
 - MXChatViewControllerTabChat: The chat tab.
 - MXChatViewControllerTabFile: The file tab.
 - MXChatViewControllerTabTodo: The todo tab.
 */
typedef NS_ENUM(NSUInteger, MXChatViewControllerTab) {
    MXChatViewControllerTabChat,
    MXChatViewControllerTabFile,
    MXChatViewControllerTabTodo
};

/**
 The MXChatViewController class provides the main chat UI components. You should specify a chat when initializing so that the view controller could know whose content it should present.
 */
@interface MXChatViewController : UIViewController

/**
 The chat whose content the view controller should present
 */
@property (nonatomic, strong) MXChat *chat;

/**
 Start date of the chat's display content.
 Configure this if you want to hide some content before the 'startDate'.
 */
@property (nonatomic, assign) NSDate *startDate;

/**
 Extra tabs of the MXChatViewController
 */
@property (nonatomic, strong, nullable) NSArray <MXChatTab *> *extraTabs;

/**
 The object acts as the delegate of this chatViewController.
 */
@property (nonatomic, readwrite, weak) id<MXChatViewControllerDelegate> delegate;


/**
 Initialize a chat view controller with a specified chat.
 
 @param chat The related chat
 @return Return a newly initialized chat view controller
 */
- (instancetype)initWithChat:(MXChat *)chat;

/**
 Switch to a specified tab with tab index. This method will do nothing if multi-tab not supported.
 
 @param tabIndex The tab at this index is going to show.
 @param animated Set YES to animate the switch progress, otherwise set NO.
 */
- (void)switchToTab:(NSUInteger)tabIndex animated:(BOOL)animated;

/**
 Scroll this chat view to the position that the specified feed is placed at the bottom.
 
 @param feed The feed will show at the bottom (Sequence id from REST API is also acceptable).
 */
- (void)scrollToFeed:(id)feed;

/**
 Open a page with the page id.
 
 @param pageId The page's sequence id from REST API.
 */
- (void)openPage:(NSString *)pageId;

/**
 Open a todo with the todo id.
 
 @param todoId The todo's sequence id from REST API.
 */
- (void)openTodo:(NSString *)todoId;


#pragma mark - Unavailable

- (instancetype)init UNAVAILABLE_ATTRIBUTE;
- (instancetype)initWithCoder:(NSCoder *)aDecoder UNAVAILABLE_ATTRIBUTE;
- (instancetype)initWithNibName:(NSString * _Nullable)nibNameOrNil bundle:(NSBundle * _Nullable)nibBundleOrNil UNAVAILABLE_ATTRIBUTE;

@end

NS_ASSUME_NONNULL_END
