//
//  MXMeet.h
//  ChatSDK
//
//  Copyright (c) Moxtra Inc. All rights reserved.
//

#import <Foundation/Foundation.h>

NS_ASSUME_NONNULL_BEGIN

@class MXUserItem;


/**
 The MXMeet class provides interfaces for basic information about meets as well as some lightweight operations.
 */
@interface MXMeet : NSObject

/**
 The id of this meet.
 */
@property (nonatomic, strong, readonly) NSString *meetId;

/**
 Topic of the meet.
 */
@property (nonatomic, strong, readonly, nullable) NSString *topic;

/**
 Host of this meet.
 */
@property (nonatomic, strong, readonly) MXUserItem *host;

/**
 A boolean value indicating whether this meet is in progress.
 */
@property (nonatomic, assign, readonly) BOOL isInProgress;

/**
 A boolean value indicating whether this is a UC meet.
 */
@property (nonatomic, assign, readonly) BOOL isUCMeet;

/**
 A boolean value indicating whether this is a recurrent meet.
 */
@property (nonatomic, assign, readonly) BOOL isRecurrent;

/**
 Start time of this meet. If the meet has not started yet, the value in this property will be nil.
 */
@property (nonatomic, strong, readonly, nullable) NSDate *startTime;

/**
 End time of this meet. If the meet has not ended yet, the value in this property will be nil.
 */
@property (nonatomic, strong, readonly, nullable) NSDate *endTime;

/**
 Scheduled start time of this meet. If the meet was not scheduled, the value in this property will be nil.
 */
@property (nonatomic, strong, readonly, nullable) NSDate *scheduledStartTime;

/**
 Scheduled end time of this meet. If the meet was not scheduled, the value in this property will be nil.
 */
@property (nonatomic, strong, readonly, nullable) NSDate *scheduledEndTime;

/**
 A boolean value indicating whether the invitation from this meet has been accepted.
 */
@property (nonatomic, assign, readonly, getter=isAccepted) BOOL accepted;

/**
 A boolean value indicating whether this meeting is declined by peer, only avaliable for a one-to-one meeting.
 */
@property (nonatomic, assign, readonly, getter=isDeclined) BOOL declined;

/**
 A boolean value indicating whether this meet is missed.
 */
@property (nonatomic, assign, readonly, getter=isMissed) BOOL missed;

/**
 The tags of this meet.
 */
@property (nonatomic, strong, readonly, nullable) NSDictionary<NSString *, NSString *>* tags;

/**
 An array containing part users of this meet.
 */
@property (nonatomic, strong, readonly) NSArray<MXUserItem *> *users;

/**
 the number of all users of this meet.
 */
@property (nonatomic, assign, readonly) NSInteger meetUserCount;

/**
 URL of this meet.
 */
@property (nonatomic, strong, readonly) NSURL *meetURL;

/**
 The recording url of this meet.
 */
@property (nonatomic, strong, readonly, nullable) NSURL *recordingUrl;

/**
 The recording name of this meet.
 */
@property (nonatomic, strong, readonly, nullable) NSString *recordingName;

/**
 The agenda of this meet.
 */
@property (nonatomic, strong, readonly, nullable) NSString *agenda;

/**
 Update topic of this meet.
 
 @param topic             The new topic.
 @param completionHandler A block to be executed when the action completes.
 @discussion A user who is not the host has no permission for updating topic, an error with code MXChatSDKErrorNoPermission will occurred if you do so.
 */
- (void)updateTopic:(NSString *)topic withCompletionHandler:(void (^ _Nullable)(NSError * _Nullable error))completionHandler;

/**
 Update agenda of this meet.
 
 @param agenda            The new agenda
 @param completionHandler A block to be executed when the action completes.
 @discussion A user who is not the host has no permission for updating agenda, an error with code MXChatSDKErrorNoPermission will occurred if you do so.
 */
- (void)updateAgenda:(nullable NSString *)agenda withCompletionHandler:(void (^ _Nullable)(NSError * _Nullable error))completionHandler;

@end

NS_ASSUME_NONNULL_END
