//
//  MXMeetListModel.h
//  ChatSDK
//
//  Copyright © 2017 moxtra. All rights reserved.
//

#import <Foundation/Foundation.h>

NS_ASSUME_NONNULL_BEGIN

@class MXMeetListModel;
@class MXMeetSession;
@class MXMeet;


/**
 The MXMeetListModelDelegate protocol describes the methods that MXMeetListModel objects call on their delegates to handle meet list events
 */
@protocol MXMeetListModelDelegate <NSObject>

@optional

/**
 Tells the delegate that some meets have been created
 
 @param meetListModel The related meet list model
 @param createdMeets  An array containing created meets
 */
- (void)meetListModel:(MXMeetListModel *)meetListModel didCreateMeets:(NSArray<MXMeet *> *)createdMeets;

/**
 Tells the delegate that some meets have been updated
 
 @param meetListModel The related meet list model
 @param updatedMeets  An array containing updated meets
 */
- (void)meetListModel:(MXMeetListModel *)meetListModel didUpdateMeets:(NSArray<MXMeet *> *)updatedMeets;

/**
 Tells the delegate that some meets have been deleted
 
 @param meetListModel The related meet list model
 @param deletedMeets  An array containing deleted meets
 */
- (void)meetListModel:(MXMeetListModel *)meetListModel didDeleteMeets:(NSArray<MXMeet *> *)deletedMeets;

/**
 Tells the delegate that a meet has been started
 
 @param meetListModel The related meet list model
 @param meet          The started meet
 */
- (void)meetListModel:(MXMeetListModel *)meetListModel didStartMeet:(MXMeet *)meet;

/**
 Tells the delegate that a meet has been ended
 
 @param meetListModel The related meet list model
 @param meet          The ended meet
 */
- (void)meetListModel:(MXMeetListModel *)meetListModel didEndMeet:(MXMeet *)meet;

@end


/**
 The MXMeetListModel class provides interfaces for creating, deleting, accepting and declining meets. It also maintains all the meets object related with current user.
 */
@interface MXMeetListModel : NSObject

/**
 The object that acts as delegate of the meet list
 */
@property (nonatomic, weak, nullable) id<MXMeetListModelDelegate> delegate;

/**
 Return an array containing all meets
 */
- (NSArray<MXMeet *> *)meets;

/**
 Start a meet by specifying meet topic
 
 @param topic             The meet topic
 @param completionHandler A block object to be exexcuted when the action completes
 @discussion An error with code MXChatSDKErrorPreconditionNotSatisfied will occurred if there is another session in progress.
 */
- (void)startMeetWithTopic:(NSString * _Nullable)topic
         completionHandler:(void (^ _Nullable)(NSError * _Nullable errorOrNil, MXMeetSession * _Nullable meetSession))completionHandler;

/**
  Start a meet by specifying meet topic and original chat

 @param topic             The meet topic
 @param chat              Meet will publish update info in this chat
 @param completionHandler A block object to be exexcuted when the action completes
 @discussion An error with code MXChatSDKErrorPreconditionNotSatisfied will occurred if there is another session in progress.
 */
- (void)startMeetWithTopic:(NSString * _Nullable)topic
                    inChat:(MXChat * _Nullable)chat
         completionHandler:(void (^ _Nullable)(NSError * _Nullable errorOrNil, MXMeetSession * _Nullable meetSession))completionHandler;

/**
 Start an individual meet with a user by specifying unique id and org id.

 @param topic             The meet topic
 @param orgId             The related user's org id which could be nil.
 @param uniqueId          The related user's unique id.
 @param completionHandler A block object to be exexcuted when the action completes
 @discussion An error with code MXChatSDKErrorPreconditionNotSatisfied will occurred if there is another session in progress.
 */
- (void)startMeetWithTopic:(NSString * _Nullable)topic
                     orgId:(NSString * _Nullable)orgId
                  uniqueId:(NSString * )uniqueId
         completionHandler:(void (^ _Nullable)(NSError * _Nullable errorOrNil, MXMeetSession * _Nullable meetSession))completionHandler;

/**
 Start an individual meet with a user by specifying unique id and org id.
 
 @param topic             The meet topic
 @param orgId             The related user's org id which could be nil.
 @param uniqueId          The related user's unique id.
 @param chat              Meet will publish update info in this chat.
 @param completionHandler A block object to be exexcuted when the action completes
 @discussion An error with code MXChatSDKErrorPreconditionNotSatisfied will occurred if there is another session in progress.
 */
- (void)startMeetWithTopic:(NSString * _Nullable)topic
                     orgId:(NSString * _Nullable)orgId
                  uniqueId:(NSString * )uniqueId
                    inChat:(MXChat *)chat
         completionHandler:(void (^ _Nullable)(NSError * _Nullable errorOrNil, MXMeetSession * _Nullable meetSession))completionHandler;

/**
 Start a meet by specifying meet id
 
 @param meetId            The meet id
 @param completionHandler A block object to be executed when the action completes
 @discussion An error with code MXChatSDKErrorPreconditionNotSatisfied will occurred if there is another session in progress.
 */
- (void)startMeetWithMeetId:(NSString *)meetId 
          completionHandler:(void (^ _Nullable)(NSError * _Nullable errorOrNil, MXMeetSession * _Nullable meetSession))completionHandler;

/**
 Join a meet by specifying meet id
 
 @param meetId            The meet id
 @param completionHandler A block object to be executed when the action completes
 @discussion An error with code MXChatSDKErrorPreconditionNotSatisfied will occurred if there is another session in progress.
 */
- (void)joinMeetWithMeetId:(NSString *)meetId
         completionHandler:(void (^ _Nullable)(NSError * _Nullable errorOrNil, MXMeetSession * _Nullable meetSession))completionHandler;

/**
 Delete meet
 
 @param meet              The meet that the user is going to delete
 @param completionHandler A block object to be executed when the action completes
 @discussion A user who is not the host has no permission for deleting the meet, an error with code MXChatSDKErrorNoPermission will occurred if you do so.
 */
- (void)deleteMeet:(MXMeet *)meet withCompletionHandler:(void (^ _Nullable)(NSError * _Nullable errorOrNil))completionHandler;


/**
 accept meet
 
 @param meetId              The meet to be accepted.
 @param completionHandler A block object to be executed when the action completes
 */
- (void)acceptMeetWithMeetId:(NSString *)meetId completionHandler:(void (^ _Nullable)(NSError * _Nullable errorOrNil))completionHandler;

/**
 decline meet
 
 @param meetId              The meet to be declined.
 @param completionHandler A block object to be executed when the action completes
 */
- (void)declineMeetWithMeetId:(NSString *)meetId completionHandler:(void (^ _Nullable)(NSError * _Nullable errorOrNil))completionHandler;

@end

NS_ASSUME_NONNULL_END
