//
//  MXMeetParticipant.h
//
//  Copyright (c) Moxtra Inc. All rights reserved.
//

#import <Foundation/Foundation.h>

NS_ASSUME_NONNULL_BEGIN

/**
 The status of the meeting's participants

 - MXMeetParticipantStatusInvited:    The participant was invited and not joined yet
 - MXMeetParticipantStatusNoResponse: The participant was invited and not joined in a period of time
 - MXMeetParticipantStatusJoined:     The participant has joined
 - MXMeetParticipantStatusLeft:       The participant has left
 */
typedef NS_ENUM(NSInteger, MXMeetParticipantState) {
    MXMeetParticipantStateInvited = 0,
    MXMeetParticipantStateNoResponse,
    MXMeetParticipantStateJoined,
    MXMeetParticipantStateLeft
};

/**
 The update type of meeting's participants
 
 - MXParticipantUpdateTypeNone:       The participant has no update
 - MXParticipantUpdateTypeVoIP:       The participant has updated VoIP status
 - MXParticipantUpdateTypeTelephony:  The participant has updated telephony status
 - MXParticipantUpdateTypeVideo:      The participant has update video status
 */
typedef NS_ENUM(NSInteger, MXMeetParticipantUpdateType) {
    MXMeetParticipantUpdateTypeNone         =  0,
    MXMeetParticipantUpdateTypeVoIP         =  1 << 0,
    MXMeetParticipantUpdateTypeTelephony    =  1 << 1,
    MXMeetParticipantUpdateTypeVideo        = 1 << 2
};

/**
  A MXMeetParticipant object defines a participant device included in a meet, consisting of the related user information and the participant device state.
 */
@interface MXMeetParticipant : NSObject

/**
 The participant id.
 */
@property (nonatomic, strong, readonly) NSString *participantId;

/**
 The unique id of the related user.
 */
@property (nonatomic, strong, readonly) NSString *uniqueId;

/**
 The org id of the related user.
 */
@property (nonatomic, strong, readonly, nullable) NSString *orgId;

/**
 A boolean value indicating whether this participant joined VoIP.
 */
@property (nonatomic, assign, readonly, getter=isInVoip) BOOL inVoIP;

/**
 A boolean value indicating whether this participant's VoIP is muted.
 */
@property (nonatomic, assign, readonly, getter=isVoipMuted) BOOL voipMuted;

/**
 A boolean value indicating whether this participant joined in video.
 */
@property (nonatomic, assign, readonly, getter=isInVideo) BOOL inVideo;

/**
 A boolean value indicating whether this participant's camera turned on.
 */
@property (nonatomic, assign, readonly, getter=isCameraOn) BOOL cameraOn;

/**
 A boolean value indicating whether this participant is in telephony.
 */
@property (nonatomic, assign, readonly, getter=isInTelephony) BOOL inTelephony;

/**
 A boolean value indicating whether this participant joins meet via pure telephony.
 @discussion YES means this participant only joins via telephony, not from computer or app.
 */
@property (nonatomic, assign, readonly, getter=isPureTelephony) BOOL pureTelephony;

/**
 A boolean value indicating whether this participant's telephony is muted.
 */
@property (nonatomic, assign, readonly, getter=isTelephonyMuted) BOOL telephonyMuted;

/**
 The status of this participant.
 */
@property (nonatomic, assign, readonly) MXMeetParticipantState state;

/**
 The update type of this participant.
 */
@property (nonatomic, assign, readonly) MXMeetParticipantUpdateType updateType;

@end

NS_ASSUME_NONNULL_END
