//
//  MXMeetSession.h
//
//  Copyright (c) Moxtra Inc. All rights reserved.
//

#import <Foundation/Foundation.h>
#import <AVFoundation/AVFoundation.h>
#import "MXMeetParticipant.h"

NS_ASSUME_NONNULL_BEGIN

@class MXMeetSession;
@class MXUserItem;

/**
 The MXMeetSessionDelegate protocol defines methods that MXMeetSession objects calls on their delegates to handle meet session events.
 */
@protocol MXMeetSessionDelegate <NSObject>

@optional
/**
 Tells the delegate that the meet session did start reconnecting.
 
 @param meetSession The meet session.
 */
- (void)meetSessionDidStartReconnecting:(MXMeetSession *)meetSession;

/**
 Tells the delegate that the meet session did reconnected.
 
 @param meetSession The meet session.
 */
- (void)meetSessionDidReconnect:(MXMeetSession *)meetSession;

/**
 Tells the delegate that the meet session did fail to reconnect.
 
 @param meetSession The meet session
 @param error Error occurred
 */
- (void)meetSession:(MXMeetSession *)meetSession didFailToReconnectWithError:(NSError *)error;

/**
 Tells the delegate that the meeting will expire after a period of time
 
 @param meetSession  The meet session
 @param remainingMinutes The remaining time before the meeting's expiration
 @param maxDuration  The maximum duration of the meeting represented with minutes
 */
- (void)meetSession:(MXMeetSession *)meetSession willExpireInMinutes:(NSUInteger)remainingMinutes maxDuration:(NSUInteger)maxDuration;

/**
 Tells the delegate that the meeting has been expired
 
 @param meetSession The meet session
 @param maxDuration The maximum duration of the meeting represented in minutes
 */
- (void)meetSession:(MXMeetSession *)meetSession didExpireWithMaxDuration:(NSUInteger)maxDuration;

/**
 Tells the delegate that some participants are invited in meet. 3rd party should send the notifications out for invitation.
 
 @param meetSession  The meet session
 @param participants The invited participants
 
 @discussion This method only notify when meet link is ready. The record movie maybe not ready.  You need try this link for later. //todo confused
 */
- (void)meetSession:(MXMeetSession *)meetSession didInviteParticipants:(NSArray<MXMeetParticipant *> *)participants;

/**
 Tells the delegate that there is a participant(include current user) has joined in meet.
 
 @param meetSession The meet session
 @param participant The participant who joined meet
 */
- (void)meetsession:(MXMeetSession *)meetSession didParticipantJoin:(MXMeetParticipant *)participant;

/**
 Tells the delegate that there is a participant(include current user) status has been changed in meet.

 @param meetSession The meet session
 @param participant The participants whose status changed
 */
- (void)meetSession:(MXMeetSession *)meetSession didParticipantUpdate:(MXMeetParticipant *)participant;

/**
 Tells the delegate that there is a participant(include current user) has left the meet.
 
 @param meetSession The meet session
 @param participant The participant who joined meet
 */
- (void)meetsession:(MXMeetSession *)meetSession didParticipantLeave:(MXMeetParticipant *)participant;


/**
 Tells the delegate that the presenter begins screen sharing. Only called in the presenter side.
 
 @param meetSession The meet session
 */
- (void)meetSessionDidStartScreenShare:(MXMeetSession *)meetSession;

/**
 Tells the delegate that the presenter ends screen sharing. Only called in the presenter side or when lost presenter.
 
 @param meetSession The meet session
 */
- (void)meetSessionDidEndScreenShare:(MXMeetSession *)meetSession;

/**
 Tells the delegate that the meet session has ended
 
 @param meetSession The meet session
 */
- (void)meetSessionDidEnd:(MXMeetSession *)meetSession;

/**
 Tells the delegate that the user has joined VoIP
 
 @param meetSession The meet session
 */
- (void)meetSessionDidJoinVoip:(MXMeetSession *)meetSession;

/**
 Tells the delegate that the user has left VoIP
 
 @param meetSession The meet session
 */
- (void)meetSessionDidLeaveVoip:(MXMeetSession *)meetSession;

/**
 Tells the delegate that the user has joined video
 
 @param meetSession The meet session
 */
- (void)meetSessionDidJoinVideo:(MXMeetSession *)meetSession;

/**
 Tells the delegate that the user has left video
 
 @param meetSession The meet session
 */
- (void)meetSessionDidLeaveVideo:(MXMeetSession *)meetSession;

@end


/**
 The MXMeetSession class provides interfaces for interacting with meet sessions as well as user interface elements required by meet.
 */
@interface MXMeetSession : NSObject

/**
 Meet id of this session
 */
@property (nonatomic, strong, readonly) NSString *meetId;

/**
 An array containing all the participants of this meeting
 */
@property (nonatomic, strong, readonly) NSArray<MXMeetParticipant *> *participantList;

/**
 The window or view that user wants to share in the meeing. The default value is nil which means the main window is shared.
 */
@property (nonatomic, strong, nullable) UIView *screenCaptureView;

/**
 A block to be executed when invite button is tapped.
 */
@property (nonatomic, copy, nullable) void (^didTapInviteButton)(MXMeetSession *session, id sender);

/**
 A block to be executed when participants' avatars are tapped.
 */
@property (nonatomic, copy, nullable) void (^didTapParticipantAvatar)(MXMeetSession *session, id sender, MXMeetParticipant *participant);

/**
 A block to be executed when end or leave button is tapped.
 */
@property (nonatomic, copy, nullable) void (^didTapEndOrLeaveMeetButton)(MXMeetSession *session, id sender);

/**
 The object acts as the delegate of this meet session.
 */
@property (nonatomic, weak, nullable) id<MXMeetSessionDelegate> delegate;


/**
 Invite users to join this meeting by specifing an array containing users to be invited.

 @param users             An array containing all users to be invited.
 @param completionHandler A block to be executed when the action completes.
 */
- (void)inviteUsers:(NSArray<MXUserItem *> *)users withCompletionHandler:(void (^ _Nullable)(NSError * _Nullable error))completionHandler;

/**
 Invite users within a organization to join this meeting by specifing org id and unique ids

 @param orgId             A string representing the org ID. Set nil if there is no one.
 @param uniqueIds         An array containing unique IDs of those who will be invited.
 @param completionHandler A block to be executed when the action completes
 */
- (void)inviteUsersWithOrgId:(NSString * __nullable)orgId
                   uniqueIds:(NSArray<NSString *> *)uniqueIds
           completionHandler:(void (^ __nullable)(NSError * __nullable errorOrNil))completionHandler;

/**
 Present the meet window.
 */
- (void)presentMeetWindow;

/**
 End(If the current user is the host) or leave the meet session.

 @param completionHandler A block to be executed when the action completes.
 @discussion An error with code MXChatSDKErrorPreconditionNotSatisfied will occurred if the session is not in progress.
 */
- (void)endOrLeaveMeetSessionWithCompletionHandler:(void (^ _Nullable)(NSError * _Nullable error))completionHandler;


/**
 Join VoIP in the meeting

 @param completionHandler A block to be executed when the action completes.
 @discussion An error with code MXChatSDKErrorPreconditionNotSatisfied will occurred if no VoIP is found in this session. An error with code MXChatSDKErrorNotSupport will occurred if the device does not support VoIP or the request for VoIP is declined by the user.
 */
- (void)joinVoIPWithCompletionHandler:(void (^ _Nullable)(NSError * _Nullable error))completionHandler;

/**
 Leave VoIP in the meeting. If the user has not joined VoIP, it will do nothing.
 */
- (void)leaveVoIP;


/**
 Start sharing a file.

 @param filePath          The local path of the file to be shared.
 @param completionHandler A block to be executed when the action completes.
 @discussion An error with code MXChatSDKErrorNoPermission will occurred if the user is not the presenter. An error with code MXChatSDKErrorPreconditionNotSatisfied will occurred if the session is not in progress.
 */
- (void)startSharingFileWithPath:(NSString *)filePath completionHandler:(void (^ _Nullable)(NSError * _Nullable error))completionHandler;

/**
 Start sharing the screen.

 @param completionHandler A block to be executed when the action completes.
 @discussion An error with code MXChatSDKErrorNoPermission will occurred if the user is not the presenter. An error with code MXChatSDKErrorPreconditionNotSatisfied will occurred if the session is not in progress.
 */
- (void)startSharingScreenWithCompletionHandler:(void (^ _Nullable)(NSError * _Nullable error))completionHandler;

/**
 Start sharing white board.

 @param completionHandler A block to be executed when the action completes.
 @discussion An error with code MXChatSDKErrorNoPermission will occurred if the user is not the presenter. An error with code MXChatSDKErrorPreconditionNotSatisfied will occurred if the session is not in progress.
 */
- (void)startSharingWhiteBoardWithCompletionHandler:(void (^ _Nullable)(NSError * _Nullable error))completionHandler;

/**
 Start video

 @param captureSource The capture device used for video.
 @param completionHandler A block to be executed when the action completes.
 @discussion An error with code MXChatSDKErrorPreconditionNotSatisfied will occurred if video is not enabled in this session. An error with code MXChatSDKErrorNotSupport will occurred if the device does not support video or the request for video is declined by the user.
 */
- (void)startVideoWithCaptureSource:(AVCaptureDevicePosition)captureSource completionHandler:(void (^ _Nullable)(NSError * _Nullable error))completionHandler;

@end

NS_ASSUME_NONNULL_END
