//
//  MXSignFileItem.h
//  ChatSDK
//
//  Copyright © 2017 moxtra. All rights reserved.
//

#import "MXFileItem.h"

NS_ASSUME_NONNULL_BEGIN

@class MXUserItem;

/**
 An enumeration for the states of sign files.
 
 - MXSignFileItemStateInvalid: A state indicating that the sign file is in an invalid state.
 - MXSignFileItemStatePreparing: A state indicating that the sign file is being prepared by the owner.
 - MXSignFileItemStateInProgress: A state indicating that the sign file is in progress.
 - MXSignFileItemStateFinished: A state indicating that the sign file has been signed by all signees.
 - MXSignFileItemStateFailed: A state indicating that the sign file has been declined.
 */
typedef NS_ENUM(NSInteger, MXSignFileItemState) {
    MXSignFileItemStateInvalid = 0,
    MXSignFileItemStatePreparing = 10,
    MXSignFileItemStateInProgress = 20,
    MXSignFileItemStateFinished = 30,
    MXSignFileItemStateFailed = 40
};

/**
 An enumeration for the states of signees.
 
 - MXFileSigneeStateInvalid: A state indicating that the user is a signee of this sign file.
 - MXFileSigneeStatePending: A state indicating that it is not the user's turn yet.
 - MXFileSigneeStateSigning: A state indicating that it is the user's turn to sign.
 - MXFileSigneeStateDeclined: A state indicating that the user has declined to sign.
 - MXFileSigneeStateSigned: A state indicaing that the user has signed this sign file.
 - MXFileSigneeStateCanceled: A state indicating that the sign file has been canceled due to someone else before the user's turn.
 */
typedef NS_ENUM(NSInteger, MXFileSigneeState) {
    MXFileSigneeStateInvalid = 0,
    MXFileSigneeStatePending,
    MXFileSigneeStateSigning,
    MXFileSigneeStateDeclined,
    MXFileSigneeStateSigned,
    MXFileSigneeStateCanceled
};


/**
 The MXSignFileItem class provides interfaces for basic information about sign files.
 */
@interface MXSignFileItem : MXFileItem

/**
 An array containing all signees of this sign file.
 */
@property (nonatomic, strong, readonly) NSArray<MXUserItem *> *allSignees;

/**
 The current signee.
 */
@property (nonatomic, strong, readonly, nullable) MXUserItem *currentSignee;

/**
 The owner of this sign file.
 */
@property (nonatomic, strong, readonly) MXUserItem *owner;

/**
 The state of this sign file.
 @see MXSignFileItemState.
 */
@property (nonatomic, readonly) MXSignFileItemState state;

/**
 Get state for a specified signee.
 
 @param signee The specified signee.
 @return A MXFileSigneeState enum representing state of the signee.
 */
- (MXFileSigneeState)stateForSignee:(MXUserItem *)signee;

/**
 Get updated time for a specified signee.
 
 @param signee The specified signee.
 @return The signee's updated time.
 */
- (NSDate *)updatedTimeForSignee:(MXUserItem *)signee;

/**
 Return the decline reason of this sign file. Return nil if this sign file is not declined.
 */
- (nullable NSString *)declineReason;

/**
 Return the time when this sign file is declined. Return nil if this sign file is not declined.
 */
- (nullable NSDate *)declineTime;

@end

NS_ASSUME_NONNULL_END
